#   Copyright (c) 1997 Boulder Real Time Technologies, Inc.           
#                                                                     
#   This software module is wholly owned by Boulder Real Time         
#   Technologies, Inc. Any use of this software module without        
#   express written permission from Boulder Real Time Technologies,   
#   Inc. is prohibited.                                               

proc station_button {parent sta staname lat lon width} { 
    set w $parent.s_$sta
    set m $w.menu
    menubutton $w -menu $m -text $sta -bg orange -width $width
    menu $m \
	-disabledforeground blue \
	-tearoff no \
	-postcommand "showMeasurements $m $sta"

    regsub -all \{ $staname "" staname
    regsub -all \} $staname "" staname
    $m add command -label $staname -state disabled
    $m add command -label ($lat,$lon) -state disabled

    $m add command -label "Latency =" -state disabled
    $m add command -label "kbaud =" -state disabled

    $m add cascade -label "Waveforms" -menu $m.wf
    menu $m.wf
    global Srcname
    global Orbname
    foreach net_sta_chan [array names Srcname] { 
	if { [regexp _${sta}_ $net_sta_chan] } {
		set l [split $net_sta_chan _]
		if {[llength $l] == 4} {
			set channel [format "%s_%s" [lindex $l 2] [lindex $l 3]]
		} else {
			set channel [lindex $l 2]
		}
		lappend all $net_sta_chan
		$m.wf add command \
		-label $channel \
		-command "exec orbmonrtd $Orbname -source $net_sta_chan &"
	}
    }
    if { [info exists all] } { 
	set all [lsort $all]
	set all [join $all " -source "]
	$m.wf add command \
	    -label All \
	    -command "exec orbmonrtd $Orbname -source $all &"
    }

    # $m add command -label "Gaps"
    # $m add command -label "Retransmissions"
    return $w
}

proc showMeasurements {menu sta} { 
    global MinLatency MaxLatency Kbaud
    if { ! [info exists MinLatency($sta)] } {
	set latency "No Data"
	set kbaud "No Data"
    } else { 
	if { $MinLatency($sta) == $MaxLatency($sta) }  { 
	    set latency $MinLatency($sta)
	} else {
	    set latency $MinLatency($sta)-$MaxLatency($sta)
	}
	set kbaud $Kbaud($sta)
    }
	
    $menu entryconfigure 2 -label "Latency = $latency"
    $menu entryconfigure 3 -label "Kbaud = $kbaud"
}


proc getStationMeasurements {} { 
# Because the orbstatus data is organized by
# srcname, and there's no one to one correlation between
# srcname and station, measurements for the station buttons
# have to be extracted 

    global Channels 
    global MinLatency MaxLatency Kbaud
    catch "unset MinLatency"
    catch "unset MaxLatency"
    catch "unset Kbaud"
    global Src
    foreach srcname [array names Channels] { 
	set channels $Channels($srcname)
	global Src$srcname
	if { [info exists Src${srcname}(latency)] } { 
	    set latency [set Src${srcname}(latency)]
	    set sta $srcname
	    regsub \[A-Za-z\]+_ $sta "" sta
	    regsub /.* $sta "" sta
	    regsub _.* $sta "" sta
	    if { ! [info exists MinLatency($sta)] 
		|| ($MinLatency($sta) > $latency) } { 
		    set MinLatency($sta) $latency 
	    }
	    if { ! [info exists MaxLatency($sta)] 
		|| ($MaxLatency($sta) < $latency) } { 
		    set MaxLatency($sta) $latency 
	    }
	    set Kbaud($sta) [set Src${srcname}(kbaud)]
	}
    }
}

proc updateNetwin {} { 
    global Orbname
    global Srcname
    global Channels

    global Src
    if { [info exists Orbname ] } {
	orbsources $Orbname Src
    }

    global MinLatency MaxLatency Kbaud
    getStationMeasurements 

    global NetworkStations
    set w $NetworkStations

    foreach sta [array names MinLatency] {
	if { [winfo exists $w.s_$sta] }  {
	    setLatencyColor $w.s_$sta $MinLatency($sta)
	}
    }
}

proc netstations {w dbsite ns size} { 

    set width [winfo width .]
    if { $width < 84 } { set width 1000 }
# There seems to be no easy way to compute screen width
# from # characters or vice versa.  With the default font, 
# and specifying a width of 10 (characters) for the menubuttons,
# the width seems to come out as 84 pixels.

    catch {unset done}
    set columns [expr int($width/84)]
    if { $columns <= 0 } {set columns 10}
    set buttonWidth 10
    set maxrow [expr floor($ns/$columns)]
    set row 0
    set col 0
    for {set i 0} {$i<$ns} {incr i} {
	set lat [dbgetv $dbsite 0 $i lat]
	set lon [dbgetv $dbsite 0 $i lon]
	set sta [dbgetv $dbsite 0 $i sta]
	set staname [dbgetv $dbsite 0 $i staname]
	if { ! [info exists done($sta)] } { 
	    set button [station_button $w $sta $staname $lat $lon $buttonWidth]
	    grid $button -row $row -column $col -stick ewns
	    if { $row >= $maxrow } { 
		set row 0
		incr col
	    } else { 
		incr row
	    }
	    set done($sta) 1
	}
    }
}

proc netwin {w} {

    frame $w

    global Pf
    set size [pfget $Pf network_canvas_size]
    global NetworkStations
    set NetworkStations $w

    global Src
    global Color
    global Channels Srcnames
    if { [info exists Orbname ] } {
	orbsources $Orbname Src
	orbchannels $Orbname Channels Srcname 
    }

    set latencyColors [pfgetlist @$Pf#latencyColors]
    foreach line $latencyColors {
	set seconds [lindex $line 0]
	set color   [lindex $line 1] 
	if { [info exists Color($color)] } { 
	    set color $Color($color)
	}
	
	if {[info exists body] } {
	    append body "elseif { \$sec < $seconds } { \n\tset bg $color } "
	} else { 
	    set body "if { \$sec < $seconds } { \n\tset bg $color } "
	}
    }
    set proc "\
	proc setLatencyColor {w sec} {\n\
	set bg $color\n\
	$body\n\
	\$w config -bg \$bg \n\
	}"
    eval $proc

    global database
    set yearday [yearday [now]]
    set db [dbopen $database r]
    set dbsite [dblookup $db 0 site 0 0 ]
    set dbsite [dbsubset $dbsite "offdate==-1 || offdate>=$yearday" ]
    set dbsite [dbsort -u $dbsite sta ]
    set n [dbquery $dbsite dbRECORD_COUNT]

    if { $n > 0 } {
	netstations $w $dbsite $n $size
    } else {
	label $w.noStations \
	    -anchor nw \
	    -justify left \
	    -text "No operating stations in $database.site" \
	    -fg red
	pack $w.noStations -side left -fill x
    }
    if { [info exists Orbname ] } {
	orbchannels $Orbname Channels Srcname 
    }
}

